Imports System
Imports System.Collections.Generic
Imports System.Diagnostics
Imports System.IO
Imports System.Text
Imports System.Reflection
Imports System.Runtime.InteropServices
Imports System.Windows.Forms
Imports Office = Microsoft.Office.Core


Partial Public Class ThisAddIn

    Friend ribbon As Bsp_Addin_Vb_Ribbon

    Protected Overrides Function RequestService(ByVal serviceGuid As Guid) As Object
        If serviceGuid = GetType(Office.IRibbonExtensibility).GUID Then
            If ribbon Is Nothing Then
                ribbon = New Bsp_Addin_Vb_Ribbon()
            End If
            Return ribbon
        End If

        Return MyBase.RequestService(serviceGuid)
    End Function

End Class


<ComVisible(True)> _
    Public Class Bsp_Addin_Vb_Ribbon
    Implements Office.IRibbonExtensibility

#Region "Ribbon-Verwaltung"
    Friend Shared ribbon As Office.IRibbonUI
    Private ucIfField As IfFieldBauen
    Public Sub New()
    End Sub

    Public Function GetCustomUI(ByVal ribbonID As String) As String Implements Office.IRibbonExtensibility.GetCustomUI
        Return GetResourceText("Bsp_Addin_VB.Bsp_Addin_Vb_Ribbon.xml")
    End Function

    Public Sub OnLoad(ByVal ribbonUI As Office.IRibbonUI)
        ribbon = ribbonUI
    End Sub

    Public Function ribbon_getImages(ByVal imageID As String) As stdole.IPictureDisp
        Return PictureConverter.ImageToPIctureDisp(My.Resources.ResourceManager.GetObject(imageID))
    End Function

    Public Sub RibbonNeuInitialisieren(ByVal doc As Word.Document)
        ribbon.Invalidate()

        Dim docTypNormal As Word.WdMailMergeMainDocType = Word.WdMailMergeMainDocType.wdNotAMergeDocument
        'Falls das aktive Dokument ein Seriendruckdokument ist,
        'die Liste der Seriendruckfelder aktualisieren
        If doc.MailMerge.MainDocumentType <> docTypNormal Then
            If Not m_MergeFieldListe Is Nothing Then m_MergeFieldListe = Nothing
            m_MergeFieldListe = New ArrayList
            m_MergeFieldListe = FieldTools.MergeFieldListeHolen(doc)
        Else
            If Not m_MergeFieldListe Is Nothing Then
                m_MergeFieldListe.Clear()
            End If
        End If
    End Sub
#End Region 'Ribbon-Verwaltung

#Region "Ribbon Callbacks"
    'Variablen fr die Anpassung einer Database-Feldfunktion
    Friend Shared InsertDatabaseToggle As Boolean = False
    Private m_MergeFieldListe As ArrayList
    Private tblDatenbank As Word.Table = Nothing
    Private fldDB As Word.Field
    Private fldNameDB As String = ""
    Private fldNameSD As String = ""
    Private isNumerisch As Boolean = False

    Public Function menu_GetContent(ByVal control As Office.IRibbonControl) As String
        Dim menuXML As String = "", q As String = Chr(34)
        menuXML = "<menu xmlns=" & q & "http://schemas.microsoft.com/office/2006/01/customui" _
        & q & "><button idMso=" & q & "EnvelopesAndLabelsDialog" & q & " /><button idMso=" _
        & q & "LabelsDialog" & q & "/></menu>"
        Return menuXML
    End Function

    Public Sub AddinMailMergeStartMenus_Click(ByVal control As Office.IRibbonControl)
        Globals.ThisAddIn.Application.CommandBars.ExecuteMso(control.Tag)
        RibbonNeuInitialisieren(Globals.ThisAddIn.Application.ActiveDocument)
    End Sub


#Region "Database-Feldfunktion anpassen"

    'Bevor die Gruppe eingeblendet wird, das Dialogfeld einblenden, um die Feldfunktion einzufgen.
    'Wird diese abgebrochen, bleibt die Gruppe unsichtbar. Ausnahme: die Markierung steht in einer Datenbankfeldfunktion.
    Public Sub ShowDataBaseGroup_Click(ByVal control As Office.IRibbonControl, ByVal isPressed As Boolean)
        InsertDatabaseToggle = isPressed
        ShowDataBaseGroup()
    End Sub

    Friend Sub ShowDataBaseGroup()
        Dim benutzerAuswahl As Integer
        Dim rng As Word.Range

        If InsertDatabaseToggle Then
            rng = Globals.ThisAddIn.Application.Selection.Range
            rng.MoveEnd(Unit:=Word.WdUnits.wdCharacter, Count:=2)
            benutzerAuswahl = Globals.ThisAddIn.Application.Dialogs(Word.WdWordDialog.wdDialogInsertDatabase).Show
            If benutzerAuswahl = 0 Then 'Es wurde abgebrochen
                Me.fldDB = DatabaseField_Holen(rng)
                If Not Me.fldDB Is Nothing Then
                    InsertDatabaseToggle = True
                Else
                    InsertDatabaseToggle = False
                End If
            Else
                Me.fldDB = DatabaseField_Holen(rng)
                If Not Me.fldDB Is Nothing Then
                    InsertDatabaseToggle = True
                Else
                    InsertDatabaseToggle = False
                End If
            End If
        End If
        ribbon.InvalidateControl("InsertDatabase")
        ribbon.InvalidateControl("ShowDatabaseGroup")
    End Sub

    'Sucht alle Feldfunktionen eines Dokuments fr eine Database-Feldfunktion ab.
    'Sofern vorhanden, wird diese zurckgegeben
    Friend Function DatabaseField_Holen(ByVal rng As Word.Range) As Word.Field
        Dim fld As Word.Field, fldDB As Word.Field = Nothing
        For Each fld In rng.Document.Fields
            If fld.Type = Word.WdFieldType.wdFieldDatabase Then
                fldDB = fld
                Exit For
            End If
        Next
        Return fldDB
    End Function

    'editBox, worin der Name des Datenbankfelds eingegeben wird, 
    'das die Tabelle mit dem Seriendruckfeld verbindet
    Public Sub DatabaseFieldName_OnChange(ByVal control As Office.IRibbonControl, ByVal text As String)
        Me.fldNameDB = text
    End Sub

    'dropDown, woraus der Name des Seriendruckfelds gewhlt wird,
    'das die Tabelle mit der Datenbank verbindet
    Public Sub MergeFieldName_Select(ByVal control As Office.IRibbonControl, ByVal selectedID As String, ByVal selectedIndex As Integer)
        Me.fldNameSD = selectedID
    End Sub

    'Aktiviert, wenn der Datentyp des verbindenden Felds nummerisch ist
    Public Sub checkNumerisch_Click(ByVal control As Office.IRibbonControl, ByVal pressed As Boolean)
        Me.isNumerisch = pressed
    End Sub

    'Stellt fest, ob Feldnamen fr die WHERE-Klausel vorhanden sind
    Public Function DBVoraussetzungen() As Boolean
        Dim ok As Boolean = True
        If fldNameDB.Length < 1 Then
            ok = False
            MessageBox.Show("Geben Sie bitte einen Feldnamen ein.")
        End If
        If fldNameSD.Length < 1 Then
            ok = False
            MessageBox.Show("Whlen Sie bitte ein Seriendruckfeld aus.")
        End If
        Return ok
    End Function
    'Aktualisiert den Feldcode der Database-Feldfunktion mit der WHERE-Klausel
    Public Sub ChangeSELECT_Click(ByVal control As Office.IRibbonControl)
        Dim fldDBCode As String = fldDB.Code.Text
        Dim startSelect As Integer = fldDBCode.IndexOf("\s")
        If startSelect = -1 Then Exit Sub

        Dim selectAnweisung As String = fldDBCode.Substring(startSelect + 4)
        Dim endSelectAnweisung As Integer = selectAnweisung.IndexOf(Chr(34))
        Dim newSelectAnweisung As String = selectAnweisung
        Dim MergeFieldName As String = Me.fldNameSD
        Dim MergeField As String = "{ Mergefield " & MergeFieldName & " }"
        Dim WhereClause As String = ""
        Dim newFldDBCode As String = ""
        newSelectAnweisung = newSelectAnweisung.Substring(0, endSelectAnweisung)
        If Not Me.isNumerisch Then
            MergeField = "'" & MergeField & "'"
        End If
        WhereClause = " WHERE [" & Me.fldNameDB & "]=" & MergeField
        If newSelectAnweisung.EndsWith("DESC") Then
            newSelectAnweisung = newSelectAnweisung.Remove(newSelectAnweisung.Length - 4) & WhereClause
        Else
            newSelectAnweisung += WhereClause
        End If
        newFldDBCode = "{ " & fldDBCode.Substring(0, startSelect + 4) & newSelectAnweisung & _
         fldDBCode.Substring(startSelect + 4 + endSelectAnweisung) & " }"
        MessageBox.Show(newFldDBCode)

        Dim dbRng As Word.Range = Me.fldDB.Result
        fldDB.Delete()
        FieldTools.FeldCodeEinfuegen(newFldDBCode, dbRng)
        Dim fld As Word.Field = Me.DatabaseField_Holen(dbRng)
        fld.Update()
    End Sub

    Public Function ShowDataBaseGroup_GetPressed(ByVal control As Office.IRibbonControl) As Boolean
        Return InsertDatabaseToggle
    End Function

    Public Function DataBaseGroup_getVisible(ByVal control As Office.IRibbonControl) As Boolean
        Select Case control.Tag
            Case "InsertDatabaseGroup"
                Return InsertDatabaseToggle
            Case Else
        End Select
    End Function

    Public Function dropDownFeldListe_getItemCount(ByVal control As Office.IRibbonControl) As Integer
        Try
            If Not m_MergeFieldListe Is Nothing Then
                Return m_MergeFieldListe.Count
            Else
                Return 0
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Function

    Public Function dropDownFeldListe_getItemLabel(ByVal control As Office.IRibbonControl, ByVal index As Integer) As String
        Return m_MergeFieldListe(index).ToString()
    End Function
#End Region 'Database-Feldfunktion anpassen

#Region "Men 'Regeln'"
    'Einsteigpunkt fr das Men 'Regeln'"
    Public Sub OnRulesButton_Click(ByVal control As Office.IRibbonControl)
        Dim wdApp As Word.Application = Globals.ThisAddIn.Application
        Select Case control.Tag
            Case "AskField"
                wdApp.Dialogs(Word.WdWordDialog.wdDialogMailMergeInsertAsk).Show()
            Case "FillinField"
                wdApp.Dialogs(Word.WdWordDialog.wdDialogMailMergeInsertFillIn).Show()
            Case "MergeRecordField"
                wdApp.ActiveDocument.Fields.Add(Range:=wdApp.Selection.Range, _
                  Type:=Word.WdFieldType.wdFieldMergeRec, PreserveFormatting:=False)
            Case "MergeSeqField"
                wdApp.ActiveDocument.Fields.Add(Range:=wdApp.Selection.Range, _
                  Type:=Word.WdFieldType.wdFieldMergeSeq, PreserveFormatting:=False)
            Case "MergeNextField"
                wdApp.ActiveDocument.Fields.Add(Range:=wdApp.Selection.Range, _
                  Type:=Word.WdFieldType.wdFieldNext, PreserveFormatting:=False)
                wdApp.ActiveWindow.View.ShowFieldCodes = False
            Case "SetField"
                wdApp.Dialogs(Word.WdWordDialog.wdDialogMailMergeInsertSet).Show()
            Case "IfField"
                AddinDialogMailMergeInsertIf(Globals.ThisAddIn.Application.ActiveDocument)
            Case Else
        End Select
    End Sub

    'Initialisiert ein Custom Task Pane mit einer Instanz des Benutzersteuerelements ucIfField.
    'Der Aufgabenbereich wird bei der Initialisierung automatisch eingeblendet.
    Friend Sub AddinDialogMailMergeInsertIf(ByVal doc As Word.Document)
        Dim ctp As Microsoft.Office.Tools.CustomTaskPane
        Me.ucIfField = New IfFieldBauen()
        ctp = Globals.ThisAddIn.CustomTaskPanes.Add(ucIfField, "Wann...Dann...Sonst")
        Me.ucIfField.IfFieldInitialisieren(doc, ctp)
        ctp.Width = 269
        ctp.Visible = True
    End Sub

#End Region 'Men Regeln

#End Region

#Region "Helpers"
    'Stellt fest, ob das gegenwrtige Dokument mit einer Datenquelle verbunden ist.
    Public Function sdAktiviert_getEnabled(ByVal control As Office.IRibbonControl) As Boolean
        Dim doc As Word.Document = Globals.ThisAddIn.Application.ActiveDocument
        Dim docTypNormal As Word.WdMailMergeMainDocType
        docTypNormal = Word.WdMailMergeMainDocType.wdNotAMergeDocument
        If doc.MailMerge.MainDocumentType = docTypNormal Then
            Return False
        Else
            Return True
        End If
    End Function

    Private Shared Function GetResourceText(ByVal resourceName As String) As String
        Dim asm As Assembly = Assembly.GetExecutingAssembly()
        Dim resourceNames() As String = asm.GetManifestResourceNames()
        For i As Integer = 0 To resourceNames.Length - 1
            If String.Compare(resourceName, resourceNames(i), StringComparison.OrdinalIgnoreCase) = 0 Then
                Using resourceReader As StreamReader = New StreamReader(asm.GetManifestResourceStream(resourceNames(i)))
                    If resourceReader IsNot Nothing Then
                        Return resourceReader.ReadToEnd()
                    End If
                End Using
            End If
        Next
        Return Nothing
    End Function

#End Region

End Class

Friend Class PictureConverter
    Inherits AxHost

    Private Sub New()
        MyBase.New(String.Empty)
    End Sub

    Public Shared Function ImageToPIctureDisp(ByVal image As Image) As stdole.IPictureDisp
        Return CType(GetIPictureDispFromPicture(image), stdole.IPictureDisp)
    End Function

    Public Shared Function IconToPictureDisp(ByVal icon As Icon) As stdole.IPictureDisp
        Return ImageToPIctureDisp(icon.ToBitmap())
    End Function

    Public Shared Function PictureDispToImage(ByVal picture As stdole.IPictureDisp) As Image
        Return GetPictureFromIPicture(picture)
    End Function
End Class