﻿Imports System.Windows.Forms
Imports System.Data
Imports Word = Microsoft.Office.Interop.Word
Imports Tools = Microsoft.Office.Tools
Imports Office = Microsoft.Office.Core

Public Class ThisDocument

#Region "Variablen auf Klassenebene"
    '''Variablen für die Benutzerschnittstelle
    Dim ap As Tools.ActionsPane         'Dokumentaktionen-Aufgabenbereich
    Dim apControls As apSteuerelemente  'Benutzerdefiniertes Steuerelement
    Friend tp As Word.TaskPane             'allgemeiner Word-Aufgabenbereich

    '''Variablen, um Speichermeldungen unterzudrücken
    Dim ThisTemplate As Word.Template  'Diese Vorlage, die dem neuen Dokument angehängt ist
    Dim NormalTemplateSaved As Boolean

    'Enthält die Daten aus der Datenbank (in diesem Beipiel, eine XML-Datei)
    Dim kundenDaten As KundenListe

    '''Variablen für das Daten-Cache
    <Cached()> Public CachedEmpfänger As String
    <Cached()> Public CachedAdresse As String
    <Cached()> Public CachedPLZ As String
    <Cached()> Public CachedOrt As String
    <Cached()> Public CachedLand As String
    <Cached()> Public CachedBetreff As String
#End Region 'Variablen auf Klassebene

#Region "ThisDocument-Ereignisse"

    Private Sub ThisDocument_Startup(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Startup
        Try
            'Die Daten einlesen
            kundenDaten = New KundenListe
            kundenDaten.ReadXml(Me.AttachedTemplate.Path & "\KundenDaten.xml")

            'Speicheraufforderungen unterbinden
            ThisTemplate = Me.AttachedTemplate
            ThisTemplate.Saved = True
            Me.Saved = True

            'Sicherstellen, dass das Dokument in der korrekten Ansicht erscheint
            Me.ActiveWindow.View.Type = Word.WdViewType.wdPrintView

            'Aufgabenbereich initialisieren und einblenden
            AufgabenBereichEin()
        Catch ex As System.Exception
            MessageBox.Show(ex.Message)
        End Try

        InitializeDataCache()
        ThisApplication.NormalTemplate.Saved = True
    End Sub

    Private Sub ThisDocument_Shutdown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shutdown

    End Sub

    Private Sub ThisDocument_BeforeSave(ByVal sender As Object, ByVal e As Microsoft.Office.Tools.Word.SaveEventArgs) Handles Me.BeforeSave
        'Daten in die Datencache schreiben
        CacheSteuerlementenDaten(CachedEmpfänger, Me.ptCC_Empfänger.InnerObject)
        CacheSteuerlementenDaten(CachedAdresse, Me.ptCC_Adresse.InnerObject)
        CacheSteuerlementenDaten(CachedPLZ, Me.ptCC_PLZ.InnerObject)
        CacheSteuerlementenDaten(CachedOrt, Me.ptCC_Ort.InnerObject)
        CacheSteuerlementenDaten(CachedLand, Me.ptCC_Land.InnerObject)
        CacheSteuerlementenDaten(CachedBetreff, Me.rtCC_Betreff.InnerObject)
    End Sub
#End Region 'ThisDocument-Ereignisse

#Region "Aufgabenbereich"
    Private Sub AufgabenBereichEin()
        'Hält fest, ob die Normal.dot nicht gespeicherte Daten enthält.
        Dim normalTemplateSaved As Boolean = True
        Dim NormalDot As Word.Template = ThisApplication.NormalTemplate
        Dim apInitialisiert As Boolean = False
        Try
            'Nach Ausführung dieser Prozedur gibt es Änderungen in der NormalTemplate.
            'Falls bislang keine vorhanden sind, die gespeichert werden sollen,
            'wird am Schluss die Abfrage nach Speicherung von Änderungen unterdrückt.
            normalTemplateSaved = NormalDot.Saved

            'Falls der Aufgabenbereich Dokumentaktionen noch nicht initialisiert wurde...
            If Me.ap Is Nothing Or Me.tp Is Nothing Then
                '...dies tun, ihm das Steuerelement zufügen, und anzeigen.
                apInitialisiert = AufgabenbereichInitialisieren()
            End If

            Me.tp.Visible = True
            'Dem Aufgabenbereich den Fokus geben
            If Me.tp.Visible Then
                ThisApplication.CommandBars("Task Pane").Controls(1).SetFocus()
                System.Windows.Forms.SendKeys.Send("{Down}")
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        Finally
            If normalTemplateSaved Then NormalDot.Saved = True
            ThisTemplate.Saved = True
        End Try

    End Sub

    Friend Function AufgabenbereichInitialisieren() As Boolean
        Dim success As Boolean = False
        Try
            Me.ap = Me.ActionsPane
            Me.apControls = New apSteuerelemente
            Me.ap.Controls.Add(apControls)
            Me.tp = ThisApplication.TaskPanes(Word.WdTaskPanes.wdTaskPaneDocumentActions)

            AufgabenbereichDatenEinbinden(apControls)

            success = True
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
        Return success
    End Function

    Private Sub AufgabenbereichDatenEinbinden(ByVal apInnerControl As Windows.Forms.Control)
        Dim tKunden As DataTable = kundenDaten.Kunde
        Dim dvKunden As New DataView(tKunden)
        Dim cboKundenListe As ComboBox
        Try
            dvKunden.Sort = "Kontaktperson"
            'Combobox mit Daten füllen
            cboKundenListe = CType(apInnerControl.Controls("KontaktpersonComboBox"), ComboBox)
            cboKundenListe.DataSource = dvKunden
            cboKundenListe.DisplayMember = "Kontaktperson"

            'Die Textboxen mit der Datenquelle verknüpfen
            apInnerControl.Controls("KontaktpersonTextbox").DataBindings.Add("Text", dvKunden, "Kontaktperson")
            apInnerControl.Controls("FirmaTextbox").DataBindings.Add("Text", dvKunden, "Firma")
            apInnerControl.Controls("StraßeTextbox").DataBindings.Add("Text", dvKunden, "Straße")
            apInnerControl.Controls("PLZTextbox").DataBindings.Add("Text", dvKunden, "PLZ")
            apInnerControl.Controls("OrtTextbox").DataBindings.Add("Text", dvKunden, "Ort")
            apInnerControl.Controls("RegionTextbox").DataBindings.Add("Text", dvKunden, "Region")
            apInnerControl.Controls("LandTextbox").DataBindings.Add("Text", dvKunden, "Land")
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub
#End Region 'Aufgabenbereich

#Region "Datencache"
    Private Sub InitializeDataCache()
        If (CachedEmpfänger Is Nothing) Then
            CachedEmpfänger = "Empfänger nicht bekannt"
        End If
        If (CachedAdresse Is Nothing) Then
            CachedAdresse = "Adresse nicht bekannt"
        End If
        If (CachedPLZ Is Nothing) Then
            CachedPLZ = "PLZ nicht bekannt"
        End If
        If (CachedOrt Is Nothing) Then
            CachedOrt = "Ort nicht bekannt"
        End If
        If (CachedLand Is Nothing) Then
            CachedLand = "Land nicht bekannt"
        End If
        If (CachedBetreff Is Nothing) Then
            CachedBetreff = "Betreff nicht bekannt"
        End If
    End Sub

    Private Function CacheSteuerlementenDaten(ByRef CacheVariable As Object, _
        ByVal cc As Word.ContentControl) As String

        Dim success As Boolean = False
        If Not cc Is Nothing Then
            Try
                CacheVariable = cc.Range.Text
                success = True
            Catch ex As Exception
                MessageBox.Show(ex.Message)
                CacheVariable = "Fehler beim Schreiben."
            End Try
        Else
            CacheVariable = "Inhaltssteuerelement nicht vorhanden."
        End If
        Return success
    End Function
#End Region


End Class
